package library.servlets;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.transaction.TransactionManager;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jboss.cache.Cache;
import org.jboss.cache.CacheManager;
import org.jboss.cache.Fqn;
import org.jboss.cache.Node;
import org.jboss.cache.notifications.annotation.CacheListener;
import org.jboss.cache.notifications.annotation.TransactionCompleted;
import org.jboss.cache.notifications.event.TransactionCompletedEvent;
import org.jboss.cache.transaction.GenericTransactionManagerLookup;

@CacheListener
public class JbossCacheListener {
    private static Log logger = LogFactory.getLog(JbossCacheListener.class);
    public static String DEFAULT_CONFIC_CACHE_NAME = "default-config-cache";
    
    private InitialContext context;
    private Cache cache;
    private TransactionManager transMgr;
    
    public boolean init(){
        try {
            this.context = new InitialContext();
            CacheManager cacheManager = (CacheManager)context.lookup("java:CacheManager");
            cache = cacheManager.getCache(DEFAULT_CONFIC_CACHE_NAME, true);
            if (cache == null) {
                logger.error("Unable to find cache " + DEFAULT_CONFIC_CACHE_NAME + ", jboss cache isn't configured properly.");
                return false;
            }
            cache.start();
            cache.addCacheListener(this);

            GenericTransactionManagerLookup tranLookup = new GenericTransactionManagerLookup();
            transMgr = tranLookup.getTransactionManager();
        } catch (NamingException e1) {
            logger.error("unable to find initial context by naming service; " + e1);
            return false;
        } catch (Exception e) {
            logger.error("unable to find initial context by naming service; " + e);
            return false;
        }


        try {
            transMgr.begin();
            loadConfig();  // load data to the cache through bunch of puts
            transMgr.commit();
        } catch (Exception e) {
            try { transMgr.rollback(); } catch(Throwable t) {}
            logger.error("Unable to load cluster configuration; " + e);
            return false;
        }


        return true;
    }
    
    @TransactionCompleted
    public void perform(TransactionCompletedEvent te)
    {
        logger.info("An trans complete event " + te.getType() + " isOriginLocal " + te.isOriginLocal() + " isSuccessful " + te.isSuccessful() + " has occured");
        if (te.getCache().getRoot() == null) {
            logger.info("cache root is null from TransactionCompletedEvent after complete.");
            return;
        }
        logger.info("transEvent: " + te.getTransaction().toString());
    }
    
    private void loadConfig() {
        Node rootNode = cache.getRoot();
        Node personRecordsNode = rootNode.addChild(Fqn.fromString("/org/mycompany/personRecords"));     
        personRecordsNode.put("name", "Peter");
        personRecordsNode.put("address", "1 main st");
    }}
